
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_CLT_XACTION_H
#define PG_CLT_XACTION_H

#include "xstd/Checksum.h"
#include "base/ObjTimes.h"
#include "runtime/httpHdrs.h"
#include "runtime/Xaction.h"

class Client;
class ServerRep;
class ContentCfg;
class DistrPoint;
class WrBuf;
class ObjWorld;

class CltXact: public Xaction {
	public:
		CltXact();

		virtual void reset();

		void enqueue();
		void exec(Client *anOwner, Connection *aConn);

		virtual void noteReadReady(int fd);
		virtual void noteWriteReady(int fd);

		virtual void cacheDistrPoint(DistrPoint *) {}
		virtual bool needRetry() const { return doRetry; }
		virtual void noteAbort() {}

		bool askedPeer() const { return thePeerState != peerUnknown; }
		bool usePeer() const { return thePeerState == peerSome; }
		void usePeer(bool doUse) { thePeerState = doUse ? peerSome : peerNone; }

		const NetAddr &nextHop() const { return theNextHop; }
		NetAddr &nextHopVar() { return theNextHop; }

	protected:
		virtual Agent *owner();
		virtual const UniqId &reqId() const;

		void makeReq(WrBuf &buf);
		Error interpretHeader();
		void noteError(const Error &err);
		void redirect();
		Size parseContent();
		void parseTag(const char *tagB, const char *tagE);

		virtual void finish(Error err);
		virtual void logStats(OLog &ol) const;

		void makeReqMethod(ostream &os);
		virtual void makeEndToEndHdrs(ostream &os);
		void makeHopByHopHdrs(ostream &os);

		virtual void consume(Size size);
		virtual void firstHandSync();

		const ObjTimes &olcTimes() const;
		bool validRelOid(const ObjId &oid) const;
		bool staleReply() const;
		void checkDateSync();
		void updatePubWorld(const ObjWorld &slice);
		Error setStatusCode(int aStatus);
		Error setViserv(const NetAddr &name, ObjId &oid) const;

	protected:
		Client *theOwner;
		ServerRep *theSrvRep;
		ContentCfg *theContentCfg;
		NetAddr theNextHop;
		RepHdr theRepHdr;
		mutable ObjTimes theOlcTimes; // cached value
		Size theConsumedSize;
		Size theReqBodySize;
		ChecksumAlg theCheckAlg;

		// this is used to check whether the xaction wants to use peers
		enum { peerUnknown = -1, peerNone, peerSome } thePeerState;

		bool doRetry;		// try the same xaction again
};

#endif
