
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "pgl/pgl.h"

#include "xstd/Assert.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglClonerSym.h"


ClonerSym::ClonerSym(SynSym &aSource, int aCount): ContainerSym("Cloner"),
	theSource(aSource.clone()), theCount(aCount) {
	Assert(theSource);
	Assert(theCount > 0);
}

ClonerSym::~ClonerSym() {
	delete theSource;
}

bool ClonerSym::isA(const String &type) const {
	if (type == "Cloner" || ContainerSym::isA(type))
		return true;

	if (const char *p = type.str("[]"))
		return theSource->isA(type(0, p - type));

	return false;
}

SynSym *ClonerSym::dupe(const String &type) const {
	if (isA(type)) {
		ClonerSym *clone = new ClonerSym(*theSource, theCount);
		return clone;
	}

	if (const char *p = type.str("[]")) {
		const String itemType = type(0, p - type);
		if (theSource->canBe(itemType)) {
			ArraySym *arr = new ArraySym(itemType);
			arr->add(*this);
			return arr;
		}
	}

	return 0;
}

int ClonerSym::count() const {
	if (theSource->isA("Container"))
		return theCount*((const ContainerSym&)theSource->cast("Container")).count();
	else
		return theCount;
}

bool ClonerSym::probsSet() const {
	if (theSource->isA("Container"))
		return ((const ContainerSym&)theSource->cast("Container")).probsSet();
	else
		return false;
}

const SynSym *ClonerSym::itemProb(int idx, double &prob) const {
	const int cnt = count();
	Assert(cnt > 0);
	Assert(0 <= idx && idx < cnt);
	if (theSource->isA("Container")) {
		const ContainerSym &c = (const ContainerSym&)theSource->cast("Container");
		const int iOffset = idx / theCount;
		const SynSym *i = c.itemProb(iOffset, prob);
		prob /= theCount;
		return i;
	} else {
		prob = 1.0/theCount;
		return theSource;
	}
}

ostream &ClonerSym::print(ostream &os, const String &pfx) const {
	return theSource->print(os, pfx) << " ** " << theCount;
}
