
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_PUBLIC_WORLD_CLASS_H
#define PG_PUBLIC_WORLD_CLASS_H

#include "xstd/Array.h"
#include "runtime/ObjWorld.h"

class ostream;


// per-polyclt public world slice for a given visible server
class PubWorldSlice: public ObjWorld {
	public:
		PubWorldSlice() {};
		PubWorldSlice(const ObjWorld &w): ObjWorld(w) {}

		void update(const ObjWorld &w);
};

// global public world for a given visible server
// not a ObjWorld child, but mimics its major interfaces
class PubWorld {
	protected:
		typedef bool (PubWorldSlice::*SliceKind)() const;

	public:
		static int PositiveWss(); // all public worlds with wss > 0

	public:
		PubWorld();
		PubWorld(const UniqId &localId);

		bool find(const UniqId &id, int &sliceIdx) const;

		// "single use" pointer or nil
		const PubWorldSlice *sliceToSync() const;
		const PubWorldSlice *newerSlice(const ObjWorld &oldSlice, int startIdx = 0) const;

		// fails if no requested slice
		const PubWorldSlice &localSlice() const;
		const PubWorldSlice &sliceAt(int idx) const;
		PubWorldSlice &sliceAt(int idx);

		bool canRepeat() const;
		bool canProduce() const;
		int size() const;
		int positiveWss() const;

		void repeat(ObjId &oid, ObjSelector *sel);
		void produce(ObjId &oid, RndGen &rng);
		void freezeWss();

		void add(const PubWorldSlice &slice);
		void store(OLog &ol) const;
		void load(ILog &il);

	protected:
		PubWorldSlice &getSlice(SliceKind kind);
		PubWorldSlice &localSlice(); // the slice for this polyclt process

	protected:
		Array<PubWorldSlice> theSlices; // all slices
		const int theLocalSlice;
		mutable int theNextSliceToSync;
};


inline
OLog &operator <<(OLog &ol, const PubWorld &pw) {
	pw.store(ol);
	return ol;
}

inline
ILog &operator >>(ILog &il, PubWorld &pw) {
	pw.load(il);
	return il;
}


extern Array<PubWorld*> ThePubWorlds;

#endif
