
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_XACTION_CLASS_H
#define PG_XACTION_CLASS_H

#include "xstd/Error.h"
#include "xstd/FileScanner.h"
#include "base/ObjId.h"
#include "runtime/Connection.h"

class IOBuf;
class Agent;

class Xaction: public FileScanUser {
	public:
		static int TheSampleDebt; // log xactions until no debt

	public:
		virtual void reset(); // XXX: split into local/virtual reset

		const UniqId &id() const { return theId; }
		const ObjId &oid() const { return theOid; }
		Connection *conn() { return theConn; }
		int logCat() const { return theLogCat; }
		bool started() const { return theStartTime > 0; }

		Size repSize() const { return theRepSize; }
		Time queueTime() const { return theEnqueTime > 0 ? theStartTime - theEnqueTime : Time::Sec(0); }
		Time lifeTime() const { return theLifeTime; }
		int httpStatus() const { return theHttpStatus; }

		virtual bool needRetry() const { return false; }

		void oid(const ObjId &anOid) { theOid = anOid; }

	protected:
		enum State { stNone = 0, stSpaceWaiting, stHdrWaiting, stBodyWaiting, stDone };

	protected:
		virtual Agent *owner() = 0;
		virtual void newState(State aState);

		void start(Connection *conn);
		virtual void finish(Error err);

		bool abortBeforeIo() const;
		bool abortAfterIo(Size size);
		bool abortIo(Connection::IoMethod m, Size *size = 0);
		virtual void abortNow();

		void stopStats();
		void logStats();
		virtual void logStats(OLog &ol) const;

		void printMsg(const IOBuf &buf) const;
		void printMsg(const IOBuf &buf, Size maxSize) const;
		void printMsg(const char *buf, Size maxSize) const;

	protected:
		static int TheCount; // to report xaction "position"

	protected:
		Connection *theConn;
		UniqId theId;

		Time theEnqueTime;  // put in a queue (optional)
		Time theStartTime;  // started doing something
		Time theLifeTime;   // [start, stop]; set in stop()
		ObjId theOid;       // various details about the transfered object
		Size theRepSize;    // reply size
		Size theReqSize;    // request size
		Size theAbortSize;  // decremented on each I/O; abort if zero
		int theHttpStatus;
		int theLogCat;      // log entry category

		State theState;
};

#endif
